@@ -29,12 +29,26 @@
 
 #define FIO_HAVE_SYSLET
 
-#define nop	__asm__ __volatile__("rep;nop": : :"memory")
+#define nop		__asm__ __volatile__("rep;nop": : :"memory")
+#define read_barrier()	__asm__ __volatile__("": : :"memory")
+#define write_barrier()	__asm__ __volatile__("": : :"memory")
 
-static inline unsigned long fio_ffz(unsigned long bitmask)
+static inline unsigned long arch_ffz(unsigned long bitmask)
 {
 	__asm__("bsfl %1,%0" :"=r" (bitmask) :"r" (~bitmask));
 	return bitmask;
 }
 
+static inline unsigned long long get_cpu_clock(void)
+{
+	unsigned long ret;
+
+	__asm__ __volatile__("rdtsc" : "=A" (ret));
+	return ret;
+}
+
+#define ARCH_HAVE_FFZ
+#define ARCH_HAVE_SSE
+#define ARCH_HAVE_CPU_CLOCK
+
 #endif
