@@ -8,7 +8,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
-#include <assert.h>
+#include <fcntl.h>
 
 #include "../fio.h"
 
@@ -16,6 +16,7 @@
 
 struct posixaio_data {
 	struct io_u **aio_events;
+	unsigned int queued;
 };
 
 static int fill_timespec(struct timespec *ts)
@@ -55,7 +56,7 @@ static int fio_posixaio_cancel(struct thread_data fio_unused *td,
 	struct fio_file *f = io_u->file;
 	int r = aio_cancel(f->fd, &io_u->aiocb);
 
-	if (r == 1 || r == AIO_CANCELED)
+	if (r == AIO_ALLDONE || r == AIO_CANCELED)
 		return 0;
 
 	return 1;
@@ -76,31 +77,43 @@ static int fio_posixaio_prep(struct thread_data fio_unused *td,
 	return 0;
 }
 
-static int fio_posixaio_getevents(struct thread_data *td, int min, int max,
-				  struct timespec *t)
+#define SUSPEND_ENTRIES	8
+
+static int fio_posixaio_getevents(struct thread_data *td, unsigned int min,
+				  unsigned int max, struct timespec *t)
 {
 	struct posixaio_data *pd = td->io_ops->data;
-	struct list_head *entry;
+	struct aiocb *suspend_list[SUSPEND_ENTRIES];
+	struct flist_head *entry;
 	struct timespec start;
-	int r, have_timeout = 0;
+	int have_timeout = 0;
+	int suspend_entries = 0;
+	unsigned int r;
 
 	if (t && !fill_timespec(&start))
 		have_timeout = 1;
 
 	r = 0;
+	memset(suspend_list, 0, sizeof(*suspend_list));
 restart:
-	list_for_each(entry, &td->io_u_busylist) {
-		struct io_u *io_u = list_entry(entry, struct io_u, list);
+	flist_for_each(entry, &td->io_u_busylist) {
+		struct io_u *io_u = flist_entry(entry, struct io_u, list);
 		int err;
 
 		if (io_u->seen)
 			continue;
 
 		err = aio_error(&io_u->aiocb);
-		if (err == EINPROGRESS)
+		if (err == EINPROGRESS) {
+			if (suspend_entries < SUSPEND_ENTRIES) {
+				suspend_list[suspend_entries] = &io_u->aiocb;
+				suspend_entries++;
+			}
 			continue;
+		}
 
 		io_u->seen = 1;
+		pd->queued--;
 		pd->aio_events[r++] = io_u;
 
 		if (err == ECANCELED)
@@ -128,10 +141,10 @@ restart:
 	}
 
 	/*
-	 * hrmpf, we need to wait for more. we should use aio_suspend, for
-	 * now just sleep a little and recheck status of busy-and-not-seen
+	 * must have some in-flight, wait for at least one
 	 */
-	usleep(1000);
+	aio_suspend((const struct aiocb * const *)suspend_list,
+							suspend_entries, t);
 	goto restart;
 }
 
@@ -145,6 +158,7 @@ static struct io_u *fio_posixaio_event(struct thread_data *td, int event)
 static int fio_posixaio_queue(struct thread_data fio_unused *td,
 			      struct io_u *io_u)
 {
+	struct posixaio_data *pd = td->io_ops->data;
 	struct aiocb *aiocb = &io_u->aiocb;
 	int ret;
 
@@ -154,15 +168,25 @@ static int fio_posixaio_queue(struct thread_data fio_unused *td,
 		ret = aio_read(aiocb);
 	else if (io_u->ddir == DDIR_WRITE)
 		ret = aio_write(aiocb);
-	else
+	else {
+#ifdef FIO_HAVE_POSIXAIO_FSYNC
 		ret = aio_fsync(O_SYNC, aiocb);
+#else
+		if (pd->queued)
+			return FIO_Q_BUSY;
 
+		do_io_u_sync(td, io_u);
+		return FIO_Q_COMPLETED;
+#endif
+	}
+		
 	if (ret) {
 		io_u->error = errno;
 		td_verror(td, io_u->error, "xfer");
 		return FIO_Q_COMPLETED;
 	}
 
+	pd->queued++;
 	return FIO_Q_QUEUED;
 }
 
@@ -173,7 +197,6 @@ static void fio_posixaio_cleanup(struct thread_data *td)
 	if (pd) {
 		free(pd->aio_events);
 		free(pd);
-		td->io_ops->data = NULL;
 	}
 }
 
@@ -201,6 +224,7 @@ static struct ioengine_ops ioengine = {
 	.cleanup	= fio_posixaio_cleanup,
 	.open_file	= generic_open_file,
 	.close_file	= generic_close_file,
+	.get_file_size	= generic_get_file_size,
 };
 
 #else /* FIO_HAVE_POSIXAIO */
