@@ -160,7 +160,7 @@ static void *hdr_priv(struct verify_header *hdr)
  * Return data area 'header_num'
  */
 static inline void *io_u_verify_off(struct verify_header *hdr,
-				    struct io_u *io_u, unsigned char header_num)
+				    struct io_u *io_u, unsigned int header_num)
 {
 	return io_u->buf + header_num * hdr->len + hdr_size(hdr);
 }
@@ -212,7 +212,7 @@ static int verify_io_u_sha256(struct verify_header *hdr, struct io_u *io_u,
 {
 	void *p = io_u_verify_off(hdr, io_u, header_num);
 	struct vhdr_sha256 *vh = hdr_priv(hdr);
-	uint8_t sha256[128];
+	uint8_t sha256[64];
 	struct sha256_ctx sha256_ctx = {
 		.buf = sha256,
 	};
@@ -260,7 +260,7 @@ static int verify_io_u_sha1(struct verify_header *hdr, struct io_u *io_u,
 }
 
 static int verify_io_u_crc7(struct verify_header *hdr, struct io_u *io_u,
-			    unsigned char header_num)
+			    unsigned int header_num)
 {
 	void *p = io_u_verify_off(hdr, io_u, header_num);
 	struct vhdr_crc7 *vh = hdr_priv(hdr);
@@ -478,7 +478,9 @@ int verify_io_u(struct thread_data *td, struct io_u *io_u)
 		hdr = p;
 
 		if (hdr->fio_magic != FIO_HDR_MAGIC) {
-			log_err("Bad verify header %x\n", hdr->fio_magic);
+			log_err("Bad verify header %x at %llu\n",
+					hdr->fio_magic,
+					io_u->offset + hdr_num * hdr->len);
 			return EILSEQ;
 		}
 
@@ -746,8 +748,10 @@ int get_next_verify(struct thread_data *td, struct io_u *io_u)
 
 		ipo = rb_entry(n, struct io_piece, rb_node);
 		rb_erase(n, &td->io_hist_tree);
+		td->io_hist_len--;
 	} else if (!flist_empty(&td->io_hist_list)) {
 		ipo = flist_entry(td->io_hist_list.next, struct io_piece, list);
+		td->io_hist_len--;
 		flist_del(&ipo->list);
 	}
 
